
from django.db.models import F
from django.utils.translation import ugettext_lazy as _
from django.contrib.admin.views.decorators import staff_member_required

from stock.forms import StockReportForm
from apps.site.views import render
from apps.products.models import Product


@staff_member_required
def get_stock_report(request):
    return _get_report(
        request,
        Product.objects.active(),
        _('Products in stock')
    )


@staff_member_required
def get_min_stock_report(request):
    return _get_report(
        request,
        Product.objects.active().filter(stock__lte=F('min_stock')),
        _('Min stock report')
    )


def _get_report(request, queryset, report_name):

    form = StockReportForm(request.GET or None)

    category = None

    if form.is_valid():
        category = form.cleaned_data['category']

    if category is not None:
        queryset = queryset.filter(category=category)

    return render(request, 'stock/report.html', {
        'report_name': report_name,
        'products': queryset,
        'form': form,
        'totals': {
            'qty': sum([p.stock for p in queryset]),
            'grand_total': sum([p.subtotal for p in queryset])
        }
    })




from django.db import models
from django.utils.translation import ugettext_lazy as _

from stock import constants


class Stock(models.Model):

    stock = models.IntegerField(_('Stock'), default=0)

    min_stock = models.IntegerField(_('Minimum stock'), default=0)

    def add_stock(self, value):

        self.validate_stock_value(value)

        self.stock += value
        self._commit_stock()

    def subtract_stock(self, value):

        self.validate_stock_value(value)

        if self.stock < value:
            raise ValueError(_('Not enough stock items'))

        self.stock -= value
        self._commit_stock()

    def validate_stock_value(self, value):

        if not isinstance(value, int):
            raise ValueError(_('Value should be integer'))

        if value < 1:
            raise ValueError(_('Value less then 1'))

    def _commit_stock(self):
        self.save(update_fields=['stock'])

    @property
    def has_stock(self):
        return self.stock > 0

    class Meta:
        abstract = True


class UnitType(models.Model):

    unit_type = models.CharField(
        _('Unit type'),
        max_length=100,
        choices=constants.UNIT_TYPES,
        default=constants.UNIT_TYPE_PCS)

    def format_qty(self, qty):
        return '{} {}'.format(qty, self.get_unit_type_display())

    class Meta:
        abstract = True




from django import forms
from django.utils.translation import ugettext_lazy as _

from categories.models import Category


class StockReportForm(forms.Form):

    category = forms.ModelChoiceField(
        empty_label=_('Choose category'),
        required=False,
        queryset=Category.objects.all())




from django.urls import path

from stock import views


app_name = 'stock'


urlpatterns = [

    path('stock-report/', views.get_stock_report,
         name='stock-report'),

    path('min-stock-report/', views.get_min_stock_report,
         name='min-stock-report')

]




from django.utils.translation import ugettext_lazy as _

UNIT_TYPE_PCS = 'pcs'

UNIT_TYPES = (
    (UNIT_TYPE_PCS, _('pcs.')),
    ('l', _('l')),
    ('ml', _('ml')),
    ('g', _('g')),
    ('kg', _('kg')),
    ('pgs', _('pgs')),
)
